<?php
namespace Opencart\Application\Controller\Catalog;
class Attribute extends \Opencart\System\Engine\Controller {
  private $error = [];

  public function index() {
    $this->load->language('catalog/attribute');

    $this->document->setTitle($this->language->get('heading_title'));

    $this->load->model('catalog/attribute');

    $this->getList();
  }

  public function add() {
    $this->load->language('catalog/attribute');

    $this->document->setTitle($this->language->get('heading_title'));

    $this->load->model('catalog/attribute');

    if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validateForm()) {
      $this->model_catalog_attribute->addAttribute($this->request->post);

      $this->session->data['success'] = $this->language->get('text_success');

      $url = '';

      if (isset($this->request->get['sort'])) {
        $url .= '&sort=' . $this->request->get['sort'];
      }

      if (isset($this->request->get['order'])) {
        $url .= '&order=' . $this->request->get['order'];
      }

      if (isset($this->request->get['page'])) {
        $url .= '&page=' . $this->request->get['page'];
      }

      $this->response->redirect($this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . $url));
    }

    $this->getForm();
  }

  public function edit() {
    $this->load->language('catalog/attribute');

    $this->document->setTitle($this->language->get('heading_title'));

    $this->load->model('catalog/attribute');

    if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validateForm()) {
      $this->model_catalog_attribute->editAttribute($this->request->get['attribute_id'], $this->request->post);

      $this->session->data['success'] = $this->language->get('text_success');

      $url = '';

      if (isset($this->request->get['sort'])) {
        $url .= '&sort=' . $this->request->get['sort'];
      }

      if (isset($this->request->get['order'])) {
        $url .= '&order=' . $this->request->get['order'];
      }

      if (isset($this->request->get['page'])) {
        $url .= '&page=' . $this->request->get['page'];
      }

      $this->response->redirect($this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . $url));
    }

    $this->getForm();
  }

  public function delete() {
    $this->load->language('catalog/attribute');

    $this->document->setTitle($this->language->get('heading_title'));

    $this->load->model('catalog/attribute');

    if (isset($this->request->post['selected']) && $this->validateDelete()) {
      foreach ($this->request->post['selected'] as $attribute_id) {
        $this->model_catalog_attribute->deleteAttribute($attribute_id);
      }

      $this->session->data['success'] = $this->language->get('text_success');

      $url = '';

      if (isset($this->request->get['sort'])) {
        $url .= '&sort=' . $this->request->get['sort'];
      }

      if (isset($this->request->get['order'])) {
        $url .= '&order=' . $this->request->get['order'];
      }

      if (isset($this->request->get['page'])) {
        $url .= '&page=' . $this->request->get['page'];
      }

      $this->response->redirect($this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . $url));
    }

    $this->getList();
  }

  protected function getList() {
    if (isset($this->request->get['sort'])) {
      $sort = $this->request->get['sort'];
    } else {
      $sort = 'ad.name';
    }

    if (isset($this->request->get['order'])) {
      $order = $this->request->get['order'];
    } else {
      $order = 'ASC';
    }

    if (isset($this->request->get['page'])) {
      $page = $this->request->get['page'];
    } else {
      $page = 1;
    }

    $url = '';

    if (isset($this->request->get['sort'])) {
      $url .= '&sort=' . $this->request->get['sort'];
    }

    if (isset($this->request->get['order'])) {
      $url .= '&order=' . $this->request->get['order'];
    }

    if (isset($this->request->get['page'])) {
      $url .= '&page=' . $this->request->get['page'];
    }

    $data['breadcrumbs'] = [];

    $data['breadcrumbs'][] = [
      'text' => $this->language->get('text_home'),
      'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'])
    ];

    $data['breadcrumbs'][] = [
      'text' => $this->language->get('heading_title'),
      'href' => $this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . $url)
    ];

    $data['add'] = $this->url->link('catalog/attribute|add', 'user_token=' . $this->session->data['user_token'] . $url);
    $data['delete'] = $this->url->link('catalog/attribute|delete', 'user_token=' . $this->session->data['user_token'] . $url);

    $data['attributes'] = [];

    $filter_data = [
      'sort'  => $sort,
      'order' => $order,
      'start' => ($page - 1) * $this->config->get('config_pagination_admin'),
      'limit' => $this->config->get('config_pagination_admin')
    ];

    $attribute_total = $this->model_catalog_attribute->getTotalAttributes();

    $results = $this->model_catalog_attribute->getAttributes($filter_data);

    foreach ($results as $result) {
      $data['attributes'][] = [
        'attribute_id'    => $result['attribute_id'],
        'name'            => $result['name'],
        'attribute_group' => $result['attribute_group'],
        'sort_order'      => $result['sort_order'],
        'edit'            => $this->url->link('catalog/attribute|edit', 'user_token=' . $this->session->data['user_token'] . '&attribute_id=' . $result['attribute_id'] . $url)
      ];
    }

    if (isset($this->error['warning'])) {
      $data['error_warning'] = $this->error['warning'];
    } else {
      $data['error_warning'] = '';
    }

    if (isset($this->session->data['success'])) {
      $data['success'] = $this->session->data['success'];

      unset($this->session->data['success']);
    } else {
      $data['success'] = '';
    }

    if (isset($this->request->post['selected'])) {
      $data['selected'] = (array)$this->request->post['selected'];
    } else {
      $data['selected'] = [];
    }

    $url = '';

    if ($order == 'ASC') {
      $url .= '&order=DESC';
    } else {
      $url .= '&order=ASC';
    }

    $data['sort_name'] = $this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . '&sort=ad.name' . $url);
    $data['sort_attribute_group'] = $this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . '&sort=attribute_group' . $url);
    $data['sort_sort_order'] = $this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . '&sort=a.sort_order' . $url);

    $url = '';

    if (isset($this->request->get['sort'])) {
      $url .= '&sort=' . $this->request->get['sort'];
    }

    if (isset($this->request->get['order'])) {
      $url .= '&order=' . $this->request->get['order'];
    }

    $data['pagination'] = $this->load->controller('common/pagination', [
      'total' => $attribute_total,
      'page' => $page,
      'limit' => $this->config->get('config_pagination_admin'),
      'url' => $this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . $url . '&page={page}')
    ]);

    $data['results'] = sprintf($this->language->get('text_pagination'), ($attribute_total) ? (($page - 1) * $this->config->get('config_pagination_admin')) + 1 : 0, ((($page - 1) * $this->config->get('config_pagination_admin')) > ($attribute_total - $this->config->get('config_pagination_admin'))) ? $attribute_total : ((($page - 1) * $this->config->get('config_pagination_admin')) + $this->config->get('config_pagination_admin')), $attribute_total, ceil($attribute_total / $this->config->get('config_pagination_admin')));

    $data['sort'] = $sort;
    $data['order'] = $order;

    $data['header'] = $this->load->controller('common/header');
    $data['column_left'] = $this->load->controller('common/column_left');
    $data['footer'] = $this->load->controller('common/footer');

    $this->response->setOutput($this->load->view('catalog/attribute_list', $data));
  }

  protected function getForm() {
    $data['text_form'] = !isset($this->request->get['attribute_id']) ? $this->language->get('text_add') : $this->language->get('text_edit');

    if (isset($this->error['warning'])) {
      $data['error_warning'] = $this->error['warning'];
    } else {
      $data['error_warning'] = '';
    }

    if (isset($this->error['name'])) {
      $data['error_name'] = $this->error['name'];
    } else {
      $data['error_name'] = [];
    }

    if (isset($this->error['attribute_group'])) {
      $data['error_attribute_group'] = $this->error['attribute_group'];
    } else {
      $data['error_attribute_group'] = '';
    }

    $url = '';

    if (isset($this->request->get['sort'])) {
      $url .= '&sort=' . $this->request->get['sort'];
    }

    if (isset($this->request->get['order'])) {
      $url .= '&order=' . $this->request->get['order'];
    }

    if (isset($this->request->get['page'])) {
      $url .= '&page=' . $this->request->get['page'];
    }

    $data['breadcrumbs'] = [];

    $data['breadcrumbs'][] = [
      'text' => $this->language->get('text_home'),
      'href' => $this->url->link('common/dashboard', 'user_token=' . $this->session->data['user_token'])
    ];

    $data['breadcrumbs'][] = [
      'text' => $this->language->get('heading_title'),
      'href' => $this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . $url)
    ];

    if (!isset($this->request->get['attribute_id'])) {
      $data['action'] = $this->url->link('catalog/attribute|add', 'user_token=' . $this->session->data['user_token'] . $url);
    } else {
      $data['action'] = $this->url->link('catalog/attribute|edit', 'user_token=' . $this->session->data['user_token'] . '&attribute_id=' . $this->request->get['attribute_id'] . $url);
    }

    $data['cancel'] = $this->url->link('catalog/attribute', 'user_token=' . $this->session->data['user_token'] . $url);

    if (isset($this->request->get['attribute_id']) && ($this->request->server['REQUEST_METHOD'] != 'POST')) {
      $attribute_info = $this->model_catalog_attribute->getAttribute($this->request->get['attribute_id']);
    }

    $this->load->model('localisation/language');

    $data['languages'] = $this->model_localisation_language->getLanguages();

    if (isset($this->request->post['attribute_description'])) {
      $data['attribute_description'] = $this->request->post['attribute_description'];
    } elseif (isset($this->request->get['attribute_id'])) {
      $data['attribute_description'] = $this->model_catalog_attribute->getDescriptions($this->request->get['attribute_id']);
    } else {
      $data['attribute_description'] = [];
    }

    if (isset($this->request->post['attribute_group_id'])) {
      $data['attribute_group_id'] = $this->request->post['attribute_group_id'];
    } elseif (!empty($attribute_info)) {
      $data['attribute_group_id'] = $attribute_info['attribute_group_id'];
    } else {
      $data['attribute_group_id'] = '';
    }

    $this->load->model('catalog/attribute_group');

    $data['attribute_groups'] = $this->model_catalog_attribute_group->getAttributeGroups();

    if (isset($this->request->post['sort_order'])) {
      $data['sort_order'] = $this->request->post['sort_order'];
    } elseif (!empty($attribute_info)) {
      $data['sort_order'] = $attribute_info['sort_order'];
    } else {
      $data['sort_order'] = '';
    }

    $data['header'] = $this->load->controller('common/header');
    $data['column_left'] = $this->load->controller('common/column_left');
    $data['footer'] = $this->load->controller('common/footer');

    $this->response->setOutput($this->load->view('catalog/attribute_form', $data));
  }

  protected function validateForm() {
    if (!$this->user->hasPermission('modify', 'catalog/attribute')) {
      $this->error['warning'] = $this->language->get('error_permission');
    }

    if (!$this->request->post['attribute_group_id']) {
      $this->error['attribute_group'] = $this->language->get('error_attribute_group');
    }

    foreach ($this->request->post['attribute_description'] as $language_id => $value) {
      if ((utf8_strlen(trim($value['name'])) < 1) || (utf8_strlen($value['name']) > 64)) {
        $this->error['name'][$language_id] = $this->language->get('error_name');
      }
    }

    if ($this->error && !isset($this->error['warning'])) {
      $this->error['warning'] = $this->language->get('error_warning');
    }

    return !$this->error;
  }

  protected function validateDelete() {
    if (!$this->user->hasPermission('modify', 'catalog/attribute')) {
      $this->error['warning'] = $this->language->get('error_permission');
    }

    $this->load->model('catalog/product');

    foreach ($this->request->post['selected'] as $attribute_id) {
      $product_total = $this->model_catalog_product->getTotalProductsByAttributeId($attribute_id);

      if ($product_total) {
        $this->error['warning'] = sprintf($this->language->get('error_product'), $product_total);
      }
    }

    return !$this->error;
  }

  public function autocomplete() {
    $json = [];

    if (isset($this->request->get['filter_name'])) {
      $this->load->model('catalog/attribute');

      $filter_data = [
        'filter_name' => $this->request->get['filter_name'],
        'start' => 0,
        'limit' => 5
      ];

      $results = $this->model_catalog_attribute->getAttributes($filter_data);

      foreach ($results as $result) {
        $json[] = [
          'attribute_id' => $result['attribute_id'],
          'name' => strip_tags(html_entity_decode($result['name'], ENT_QUOTES, 'UTF-8')),
          'attribute_group' => $result['attribute_group']
        ];
      }
    }

    $sort_order = [];

    foreach ($json as $key => $value) {
      $sort_order[$key] = $value['name'];
    }

    array_multisort($sort_order, SORT_ASC, $json);

    $this->response->addHeader('Content-Type: application/json');
    $this->response->setOutput(json_encode($json));
  }
}
